#!/usr/bin/python
# -*- coding: latin-1 -*-

"""
cloudPlayer
The cloud Player composition module. 

Playback of cloud parameter automation data read from file or memory preset.

@author: yvind Brandtsegg
@contact: obrandts@gmail.com
@license: GPL
"""

import os
import util
import cloudParameters
import copy

class CloudPlayer:    
    """
    Playback of cloud parameter automation data read from file or memory preset.
    """
    
    def __init__(self):
        """
        Class constructor.
        
        @param self: The object pointer.
        """

        self.loadedPresets = []
        self.cloudSequenceContinueFlag = 0
    	self.cloudDuration = 3.0
        self.parameters = copy.copy(cloudParameters.parameterFullnamesAndDefaults) # read parameter list and default values from file
        self.parameterBasenames = copy.copy(cloudParameters.parameterBasenames)

    def getCloudSegmentValues(self, parameter, segment):
        """
        Get the values for a single breakpoint segment for a single parameter. Values stored in self.parameters
        
        @param self: The object pointer.
        @param parameter: A valid parameter "prefix", e.g. TimeposRdev. A suffix, e.g. Start, will be appended to format the full parameter string.
        @param segment: Sets the segment type to "first" or "second" (each cloud preset has a breakpoint style automation consisting of 2 segments).
        """
        if segment == 'first':
            startstring = 'Start'
            timestring = 'Time1'
            curvestring = 'Curve1'
            endstring = 'Middle'
        elif segment == 'second':
            startstring = 'Middle'
            timestring = 'Time2'
            curvestring = 'Curve2'
            endstring = 'End'
        else:
            print 'cloudSegment: invalid segment type'
            return
        startparameter = parameter + startstring
        timeparameter = 'v1Amp' + timestring        # all segment times controlled by same time parameter
        if parameter.find('Rdev') > 0:          # if the parameter string contains the substring 'rDev'
            timeparameter = 'v1AmpRdev' + timestring
        curveparameter = parameter + curvestring
        endparameter = parameter + endstring
        start = self.parameters[startparameter]
        duration = self.parameters[timeparameter]
        curve = self.parameters[curveparameter]
        end = self.parameters[endparameter]

        # size and turbulence mapping
        duration = duration * self.cloudDuration
        delta = duration

        # get mapping type for parameter
        maptype = self.getMappingType(parameter)

        return start, duration, curve, end, maptype

    def getMappingType(self, parameter):
        """
        Get mapping type for a  specified parameter in the cloud.
        
        @param self: The object pointer
        @param parameter: A valid parameter "prefix", e.g. TimeposRdev. 
        """
        # mapping type, for cloud modification mapping in csound
        maptype = 0
        if parameter.find('Rdev') > 0:
            maptype = 1
        elif parameter.find('Amp') > 0:
            maptype = 2
        elif parameter.find('GrRate') > 0:
            maptype = 3
        elif parameter.find('Transp') > 0:
            maptype = 4
        elif parameter.find('GrDur') > 0:
            maptype = 5
        elif parameter.find('grDist') > 0:
            maptype = 6
        elif parameter.find('randMask') > 0:
            maptype = 7
        if parameter.find('chanMask') > -1:
            maptype = 8
            
        return maptype

    def loadCloudPresetsFromFile(self):
        """
        Search for all cloud preset files in directory. Load presets into memory and update GUI.
        
        @param self: The object pointer.
        """
        files = os.listdir('presets')
        for f in files:
            self.loadSingleCloudPresetFromFile(f)
        for presetName in self.loadedPresets:
            print 'loaded', presetName
    
    def loadSingleCloudPresetFromFile(self, filename):
        """
        Load a single cloud preset from file.
        
        @param self: The object pointer.
        @param filename: The file name of the preset to load.
        """
        if filename.find('.clo') > 0:
            presetName = filename.rstrip('.clo')
            if presetName not in self.loadedPresets:
                self.loadedPresets.append(presetName)
            dictNameString = 'self.presetDict_'+presetName    # format the name of the dictionary to hold the preset file
            exec(dictNameString+'={}') # execute the string as a normal python statement
            codeObject = compile(dictNameString, 'string', 'eval')
            dictName = eval(codeObject)
            fil = file('presets/'+filename)
            for line in fil.readlines():
                try:
                    parameter, value = line.split(', ') #unpack parameter name and value
                    dictName[parameter]= float(value) # update dictionary
                except:
                    s = line.split(',')
                    parameter = s[0]
                    values = util.makeListFromString(line)
                    dictName[parameter]= values     # update dictionary
                    
            # check if the loaded preset lacks any of the parameters needed (can happen if new cloud play parameters have been added after the preset was saved)
            for p,v in cloudParameters.parameterFullnamesAndDefaults.iteritems():
                if p in dictName.keys():
                    continue
                else:
                    dictName[p] = v # add parameter with default value
                
            print 'loaded', presetName
            return presetName

    def updateSingleCloudPreset(self, presetName):
        """
        Updates internal stored preset from current self.parameters, done when saving to file
        
        @param self: The object pointer.
        @param presetName: The name of the preset.
        """
        isNewPreset = False
        presetName = presetName.rstrip('.clo')
        if presetName not in self.loadedPresets:
            self.loadedPresets.append(presetName)
            isNewPreset = True
        dictNameString = 'self.presetDict_'+presetName    # format the name of the dictionary to hold the preset file
        exec(dictNameString+'={}') # execute the string as a normal python statement
        codeObject = compile(dictNameString, 'string', 'eval')
        dictName = eval(codeObject)

        for parameter,value in self.parameters.iteritems():
            try:
                dictName[parameter]= float(value) # update dictionary
            except:
                dictName[parameter]= list(value)     # update dictionary            
        print 'updated copy in memory', presetName
        return isNewPreset
        
    def makePresetCurrent(self, presetName):
        """
        Make a named preset the current setting for cloud parameters by copying the values into self.parameters
        
        @param self: The object pointer.
        @param presetName: The name of the preset to recall.
        """
        presetDict = eval('self.presetDict_'+presetName)
        for p,v in presetDict.iteritems():
            self.parameters[p] = v
        # print 'current cloud preset:', presetName