#!/usr/bin/python
# -*- coding: latin-1 -*-

""" 
This is an interface module for communication from Python to Csound.
All note events messages sent to csound are collected here.

@author: yvind Brandtsegg
@contact: obrandts@gmail.com
@license: GPL
@requires: csnd
"""

import csnd

class CsoundMessages:
    """This class handles all communication between Python and the Csound engine"""

    def __init__(self, csound):
        """
        Class constructor.
        
        @param self: The object pointer.
        @param csound: Instance of Csound interface object.
        """
        self.csound = csound
        
    def csoundEventGeneric(self, pfields):
        """
        Generic access to sending events to csound,
        all p-fields must be valid and in correct order,
        no checking or processing done here

        @param self: The object pointer.
        @param pfields: The pfileds to the csound event.
        """
        strPfields = 'i'
        for item in pfields:
            strPfields = '%s %f' %(strPfields, item)
        self.csound.InputMessage(strPfields)
    
    def csoundCloudSegment(self, parameter, start, duration, curve, end, maptype):
        """
        Instantiate an instr event, generating a curved segment for parameter control.
        Each cloud parameter is controlled separately.

        @param self: The object pointer.
        @param parameter: The parameter number.
        @param start: The start value for the parameter.
        @param duration: The duration of the automation segment.
        @param curve: The curve for the automation segment (linear, convex, concave, see csound's transeg opcode).
        @param end: The end value for the parameter.
        @param maptype: The mapping type for the parameter.
        """
        instr = 101
        #print instr, duration, start, curve, end, parameter, maptype
        self.csound.InputMessage('i %i 0 %f %f %f %f "%s" %i' %(instr, duration, start, curve, end, parameter, maptype))

    def cloudParameterMapping(self, scoreLength, parameter, maptype):
        """
        Modification of cloud parameters, realtime mapping of metaparameters affecting each individual cloud parameter

        @param self: The object pointer.
        @param scoreLength: The total length of the csound score (duration for always on instrument)
        @param parameter: The parameter number.
        @param maptype: The mapping typoe for this parameter.
        """
        instr = 102
        self.csound.InputMessage('i %i 0 %i "%s" %i' %(instr, scoreLength, parameter, maptype))
    
    def setTableValues(self, tableNum, size, alist):
        """
        Set the values in a csound ftable

        @param self: The object pointer.
        @param tableNum: The table number.
        @param size: The table size.
        @param alist: The list of values to be put into the table, will be zero padded if shorter than size.
        """
        while len(alist) < size:
            alist.append(0)
        i = 0
        while i < size:
            self.csound.TableSet(tableNum, i, alist[i])
            i += 1
    
    def GetChannel(self, channel):
        """
        Wrapper for csoundAPI GetChannel() .
        
        @param self: The object pointer.
        @param channel: The channel to query.
        """
        return self.csound.GetChannel(channel)
    
    
    def getCsoundBusChannels(self):
        """
        Return a list of all allocated bus channels between python and csound

        @param self: The object pointer.
        """
        chnlst = csnd.CsoundChannelList(self.csound)
        channelList = []
        n = chnlst.Count()
        for i in range(n):
            item = []
            item.append(chnlst.Name(i))
            if chnlst.IsControlChannel(i):
                item.append('control')
            elif chnlst.IsAudioChannel(i):
                item.append('audio')
            elif chnlst.IsStringChannel(i):
                item.append('string')
            channelList.append(item)
        return channelList
        
    def listCsoundBusChannels(self):
        """
        Print a list of all allocated bus channels between python and csound

        @param self: The object pointer.
        """
        chnlst = csnd.CsoundChannelList(self.csound)
    
        n = chnlst.Count()
        if n > 0:
            print '============================================================'
            print 'Csound chn channels listing'
            print 'Currently using', n, 'channels'
            print '------------------------------------------------------------'
        for i in range(n):
            print 'Name:', chnlst.Name(i)
            if chnlst.IsControlChannel(i):
                print '       | Type: control', '| Input:', ['no', 'yes'][chnlst.IsInputChannel(i)], '| Output:', ['no', 'yes'][chnlst.IsOutputChannel(i)]
            elif chnlst.IsAudioChannel(i):
                print '       | Type: audio', '| Input:', ['no', 'yes'][chnlst.IsInputChannel(i)], '| Output:', ['no', 'yes'][chnlst.IsOutputChannel(i)]
            elif chnlst.IsStringChannel(i):
                print '       | Type: string', '| Input:', ['no', 'yes'][chnlst.IsInputChannel(i)], '| Output:', ['no', 'yes'][chnlst.IsOutputChannel(i)]
    
    def getChnValue(self, name):
        """
        Get the value stored in a csound chn channel
        
        @param self: The object pointer.
        @param name: The name of the channel to be interrogated.
        """
        return self.csound.GetChannel(name)
    
    def sendControlToCsound(self, parameter, value):
        """
        Send a control value to csound, using the software bus and chn opcode in csound

        @param self: The object pointer.
        """
        channelName = parameter
        channel = " "   # empty, for testing if channel excists
    
        # read channel list, find channel name 
        for item in self.getCsoundBusChannels():
            if item[0] == channelName:
                channel = channelName
        if channel == " ":
            print 'csound bus channel name not found'
        else:
            # send value to channel
            self.csound.SetChannel(channel, value)
            print 'sent control', channel, value
    
    def createAudioFileFtable(self, soundFile):
        """
        Create a csound ftable, loading an audio file. The table will have the smallest power-of-two size that fits the audio file
        
        @param self: The object pointer.
        @param soundFile: The name of the sound file to be loaded.
        """
        instrNum = 5
        self.csound.InputMessage('i %i 0 1 0 "%s"' %(instrNum, soundFile))
