#!/usr/bin/python
# -*- coding: latin-1 -*-

"""
eventCaller

This is a "glue" module, collecting inputs from both GUI and the timed queue,
throughputting function calls to the various modules.
It seemed tidier to keep all the imports and references to various modules in one place,
instead of cluttering up all other modules

@author: yvind Brandtsegg
@contact: obrandts@gmail.com
@license: GPL
"""

import timedEvents
import cloudPlayer
import time
import copy
import os

class EventCaller:
    """
    Functions for calling other functions.
    This is a very central "glue" module that communicates with GUI/Time/Queue/other inputes,
    and forwards properly formatted function calls to the underlying "worker" modules.
    """

    def __init__(self, csMessages):
        """
        Class constructor.
        
        @param self: The object pointer.
        @param csMEssages: Pointer to csMEssages, communicating with csound.
        """
        self.csMessages = csMessages

        self.theTimeSeconds = timedEvents.TheTime(self) 
        """The timed queue for automation playback."""
        
        self.cloudPlayer = cloudPlayer.CloudPlayer()
        """Instance of the CloudPlayer class."""

        self.guiCloudPreset = ''
        """Pointer to cloudPreset gui panel."""
        self.guiPlayControls = ''
        """Pointer to playControls gui panel."""

    def updateCloudPresetGui(self):
        """
        Update gui widgets for cloud parameter values.
        
        @param self: The object pointer.
        """
        i = 0
        for p,v in self.cloudPlayer.parameters.iteritems(): 
            #print i, p,v
            # update GUI
            ctrlString = 'self.guiCloudPreset.' + p + 'Ctrl.ChangeValue(\"' + str(v) + '\")'
            ctrlFunc = compile(ctrlString, 'string', 'eval')
            eval(ctrlFunc)
            i += 1
        print i, 'gui items updated'

    def playCloud(self, segment='first'):
        """
        Playback of the currently loaded particle cloud automation in cloudPlayer
        
        @param self: The object pointer    
        @param segment: The automation segment, may be 'first' or 'second'.
        """
        #print '*** playCloud ***', segment
        for p in self.cloudPlayer.parameterBasenames:
            # play the cloud parameter automation
            self.cloudSegment(p, segment)
            if 'chanMask' not in p:
                rDevParameter = p + 'Rdev'
                self.cloudSegment(rDevParameter, segment)

    def cloudSegment(self, parameter, segment):
        """
        Starts a timed process generating one segment of cloud automation at a time.
        
        @param self: The object pointer.
        @param parameter: A valid parameter "prefix", e.g. TimeposRdev. A suffix, e.g. Start, will be appended to format the full parameter string.
        @param segment: Sets the segment type to "first" or "second" (each cloud preset has a breakpoint style automation consisting of 2 segments).
        """
        instr = 1201
        start, duration, curve, end, maptype = self.cloudPlayer.getCloudSegmentValues(parameter, segment)
        if maptype == 8: # special mapping for channel masks, as the data is a list (contents of csound ftable)
            table1 = int(self.csMessages.GetChannel("chnMaskCloudTemp1"))
            table2 = int(self.csMessages.GetChannel("chnMaskCloudTemp2"))
            tableSize = 8
            self.csMessages.setTableValues(table1, tableSize, start)
            self.csMessages.setTableValues(table2, tableSize, end)
            start = 0.0
            end = 1.0
        #print 'cloudSegment:', segment, parameter, parameterNum, start, end
        self.csMessages.csoundCloudSegment(parameter, start, duration, curve, end, maptype)
        delta = duration
        if segment == 'first':
            # schedule another event in the timed queue
            #print 'cloudSegment inserted event into theTimeSeconds:', parameter
            self.theTimeSeconds.insertQueueWithOffset(delta, ['cloudSegment', delta, parameter, 'second'])
        
    def cloudSequencePlay(self):
        """
        Starts a timed process playing back a sequence of particle cloud automation presets
        
        @param self: The object pointer.
        """
        # if the auto continue flag is set,
        if (self.cloudPlayer.cloudSequenceContinueFlag == 1):
            try:
                cloudData = self.cloudSequenceList.pop(0)
                #print '***debug cloudSequence:', cloudData, 'at time:', time.localtime()
                #print 'cloud sequence after pop', self.cloudSequenceList
                try:
                    cloudPreset, duration = cloudData.split(', ') # it's a string, so needs to be unpacked this way
                    duration = float(duration)
                    self.cloudPlayer.cloudDuration = duration
                    if cloudPreset == 'interpolate':
                        self.cloudValuesInterpolateFromCurrentToNextPresetStart() # load the interpolate preset
                        self.playCloud(segment='second') # interpolate preset only uses the second segment
                    elif cloudPreset == 'static':
                        self.cloudValuesStaticPreset() # load the static preset
                        self.playCloud(segment='second') # static preset only uses the second segment
                    else:
                        self.cloudPlayer.makePresetCurrent(cloudPreset)
                        self.playCloud()
                    # schedule another event in the timed queue
                    #print 'inserting sequence event in queue'
                    self.theTimeSeconds.insertQueueWithOffset(duration, ['cloudSequence'])
                    # update gui
                    self.guiPlayControls.updateCloudSequenceList(self.cloudSequenceList)
                except:
                    print 'eventCaller.cloudSequencePlay: check format of item "%s" in cloud sequence list' %cloudData
                    print 'skipping to next cloud sequence event'
                    self.cloudSequencePlay()
            except:
                # insert a new cloudSequencePlay event in the timed queue, waiting a few seconds for cloudData
                #print 'no items in cloudSequenceList at time:', time.localtime()
                waitTime = 5
                self.theTimeSeconds.insertQueueWithOffset(waitTime, ['cloudSequence'])
                cloudData = None


    def cloudSequenceSave(self, cloudSequenceList, filename):
        """
        Save a sequence (list) of particle cloud automation presets.
        
        @param self: The object pointer.
        """
        f = file(filename, 'w')
        for line in cloudSequenceList:
            f.write(line+'\n')
        f.close()

    def cloudSequenceRecall(self, filename):
        """
        Recall a sequence (list) of particle cloud automation presets.
        
        @param self: The object pointer.
        """
        f = file(filename, 'r')
        cloudData = []
        for line in f.readlines():
            line = line.splitlines(0) # get rid of newline character            
            cloudData.append(line[0]) # use indexing [0] because the type was changed to list during splitlines
        self.cloudSequenceList = cloudData
        self.guiPlayControls.updateCloudSequenceList(cloudData)
        f.close()

    def setCloudSequenceContinueFlag(self, value):
        """
        Sets the autocontinue flag for the process playing back a sequence of particle cloud automation presets.
        
        @param self: The object pointer.
        @parame value: The value of the flag.
        """
        self.cloudPlayer.cloudSequenceContinueFlag = value

    def cloudValuesInterpolateFromCurrentToNextPresetStart(self):
        """
        A special internal "preset": Interpolate from the current value to the start value of the next preset to be called in a cloud sequence.
        
        Get the current values for all automation channels used in the particle cloud player.
        Store the values as 'Middle' values of the current cloud preset.
        Get the starting values for the next preset in the cloud sequence list.
        Store these values as 'End'  values of the current cloud preset.
        The interpolation can be executed by calling self.cloudPlaySegment('second')
        
        @param self: The object pointer.
        """
        # find the name of the next preset and check if it's valid
        alist = copy.copy(self.cloudSequenceList)
        # the following should be moved to cloudPlayer ###
        #print 'current cloud preset: interpolate'
        try:
            nextPreset = alist.pop(0) # find the first in the list
            nextPresetName, dur = nextPreset.split(', ') # it's a string, so has to be unpacked this way
            self.cloudPlayer.loadedPresets.index(nextPresetName) # if it's' a valid preset name, this operation will not result in an error
        except:
            print 'eventCaller.cloudValuesInterpolateFromCurrentToNextPresetStart: no next preset in cloud sequence list, or preset name not found'
            print 'interpolation automation failed'
            return
        # get end values for current preset and start values of the next preset, update cloud parameter values accordingly
        for p in self.cloudPlayer.parameterBasenames:
            if 'v1Amp' in p:
                self.setCloudParameterValue(p+'Time2', 1.0)
                self.setCloudParameterValue(p+'RdevTime2', 1.0)
            self.setCloudParameterValue(p+'Middle', self.cloudPlayer.parameters[p+'End'])
            self.setCloudParameterValue(p+'Curve2', 0.0)
            self.setCloudParameterValue(p+'End', eval('self.cloudPlayer.presetDict_'+nextPresetName+'["'+p+'Start"]'))
            if 'chanMask' not in p:
                self.setCloudParameterValue(p+'RdevMiddle',self.cloudPlayer.parameters[p+'RdevEnd'])
                self.setCloudParameterValue(p+'RdevCurve2', 0.0)
                self.setCloudParameterValue(p+'RdevEnd', eval('self.cloudPlayer.presetDict_'+nextPresetName+'["'+p+'RdevStart"]'))

    def cloudValuesStaticPreset(self):
        """
        A special internal "preset": A static cloud preset, using the current values as both start and end. Used as a "hold" or "wait" type cloud in a sequence.
        
        Get the current values for all automation channels used in the particle cloud player.
        Store the values as 'Middle' and 'End' values of the current cloud preset.
        The static preset is executed by calling self.cloudPlaySegment('second')
        
        @param self: The object pointer.
        """
        # the following should be moved to cloudPlayer ###
        #print 'current cloud preset: static'
        # get current values of the chn channels and start values of the next preset, update cloud parameter values accordingly
        for p in self.cloudPlayer.parameterBasenames:
            if 'v1Amp' in p:
                self.setCloudParameterValue(p+'Time2', 1.0)
                self.setCloudParameterValue(p+'RdevTime2', 1.0)
            self.setCloudParameterValue(p+'Middle', self.cloudPlayer.parameters[p+'End'])
            self.setCloudParameterValue(p+'Curve2', 0.0)
            if 'chanMask' not in p:
                self.setCloudParameterValue(p+'RdevMiddle',self.cloudPlayer.parameters[p+'RdevEnd'])
                self.setCloudParameterValue(p+'RdevCurve2', 0.0)


    def updateGuiWithCloudPresetNames(self):
        """
        Update the menu object in the Gui to reflect the presets in memory
        
        @param self: The object pointer.
        """
        self.guiPlayControls.updateCloudPresetComboBox(self.cloudPlayer.loadedPresets)
   
    def cloudWaveSelect(self, waveNum, waveName):
        """
        Select the waveform(s) used for particle clouds.
        
        @param self: The object pointer.
        @param waveNum: The wave number (1,2,3,4) to be set.
        @param waveName: The name of the waveform to be selected.
        """
        chnName = "particleWave"+str(waveNum)
        #print 'eventCaller.cloudWaveSelect', chnName, self.csMessages.getChnValue(waveName)
        self.csMessages.sendControlToCsound(chnName, self.csMessages.getChnValue(waveName))
        self.guiPlayControls.updateCloudWave(waveNum, waveName)

    def getCloudWavefileNames(self):
        """
        Get a list of the sound files in the "samples/particleWaves" directory.
        
        @param self: The object pointer.
        """
        files = os.listdir('samples')
        # check that they are valid wave file names
        for f in files:
            if f.find('.wav') == -1:
                files.remove(f) 
        return files

    def setCloudDuration(self, duration):
        """
        Sets the Cloud duration for the cloud player.
        
        @param self: The object pointer.
        @param duration: the cloud duration.
        """
        self.cloudPlayer.cloudDuration = duration

    def setCloudParameterValue(self, parameterName, value):
        """
        Sets individual parameter values in the cloudPlayer current preset.
        
        @param self: The object pointer.
        @param parameterName: The parameter of the cloud automation.
        @parame value: The value to set the parameter to.
        """
        self.cloudPlayer.parameters[parameterName] = value

    def sendControlToCsound(self, name, value):
        """
        Wrapper function for sending controls to Csound from GUI or other interfaces.
        
        @param self: The object pointer.
        @param name: The name of the csound chn channel
        @param value: The new value of the parameter.
        """
        self.csMessages.sendControlToCsound(name, value)

    def initValues1(self):
        """
        Initializes various values.
        
        @param self: The object pointer.
        """
        # initiate automation processes (always on)
        print 'waiting for disk read of presets and data'
        self.cloudPlayer.loadCloudPresetsFromFile()
        self.updateGuiWithCloudPresetNames()

    def initValues2(self):
        """
        Initializes values that needs to be initialized after csound has started.
        
        @param self: The object pointer.
        """
        # get a list of sound files to load
        filelist = self.getCloudWavefileNames()
        self.guiPlayControls.updateCloudWaveComboBox(filelist)
        # load sound files for partikkel
        for f in filelist:
            if f.find('.wav') >= 0:
                self.csMessages.createAudioFileFtable(f)
                while self.csMessages.GetChannel(f) == 0:   # wait until csound is done loading the sound file before doing anything else
                    time.sleep(0)
        # init particleWave chn channels
        self.cloudWaveSelect(1, f)
        self.cloudWaveSelect(2, f)
        self.cloudWaveSelect(3, f)        
        self.cloudWaveSelect(4, f)

        scoreLength = 60000 # 
        for p in self.cloudPlayer.parameterBasenames:
            maptype = self.cloudPlayer.getMappingType(p)
            self.csMessages.cloudParameterMapping(scoreLength, p, maptype) # parameter mapping instr always on
            if 'chanMask' not in p:
                rDevParameter = p + 'Rdev'
                maptype = self.cloudPlayer.getMappingType(rDevParameter)
                self.csMessages.cloudParameterMapping(scoreLength, rDevParameter, maptype) # parameter mapping instr always on

        print 'turning on csound instruments'
        time.sleep(1)
        self.csMessages.csoundEventGeneric([103, 0, scoreLength, 4]) # 4 voice partikkel
        self.csMessages.csoundEventGeneric([301, 0, scoreLength]) # efx: chorus
        self.csMessages.csoundEventGeneric([302, 0, scoreLength]) # efx: reverb
        #self.csMessages.csoundEventGeneric([9998, 0, scoreLength]) # output file write
        self.csMessages.csoundEventGeneric([9999, 0, scoreLength]) # audio output

    def parseEvent(self, event):
        """
        Parsing of events output from timed queue.

        @param self: The object pointer.
        @param event: The event to be processed.
        """
        eventType = event[0]
        data = event[1:]
        if eventType == 'csoundEvent':
            print 'not implemented ? '
            #eventCaller1.csoundEventGeneric(data)
        elif eventType == 'cloudSegment':
            parameter, segment = data[1:]
            self.cloudSegment(parameter, segment)
        elif eventType == 'cloudSequence':
            self.cloudSequencePlay()
        else:
            print 'event in timed queue of unknown type', event
