#!/usr/bin/python
# -*- coding: latin-1 -*-

"""
playControls

The gui panel with widgets for playback of cloud parameter automation.
It also includes some metaparameters for modulation of the playback.

@author: yvind Brandtsegg
@contact: obrandts@gmail.com
@license: GPL
@requires: wxWidgets for Python
"""


import wx
import  wx.gizmos
import cloudGui
import util
import os

csound = "global csound instance"
"""Pointer to the global csound instance."""

#Gui widget IDs
ID_Play = 1 
ID_Duration = 2
ID_masterAmp = 3
ID_transpScale = 4
ID_rateScale = 5
ID_Turbulence = 6
ID_Transparency = 7

ID_AddToSeq = 101
ID_loadWithList = 124
ID_cloudPresetMenu = 125
ID_cloudSequence = 126
ID_cloudSequencePlay = 127
ID_cloudReverb = 129
ID_cloudDelay = 130
ID_cloudWidth = 131

ID_cloudWave1 = 132 
ID_cloudWave2 = 133
ID_cloudWave3 = 134
ID_cloudWave4 = 135

ID_ShowCloudPresetControls = 140

# menubar items IDs
ID_FileMenuOpenCloud = 201
ID_FileMenuSaveCloud = 202
ID_FileMenuSaveCloudAs = 203
ID_FileMenuOpenSeq = 204
ID_FileMenuSaveSeq = 205
ID_FileMenuSaveSeqAs = 206
ID_FileMenuExit = 207

class PlayControls(wx.Frame):
    """
    The gui panel with widgets for playback of cloud parameter automation.
    """

    def __init__(self, parent, eventCaller):
        """
        Class constructor.
        
        @param self: The object pointer.
        @param parent: The window parent.
        @param eventCaller: Pointer to the eventCaller instance.
        """
        # call the parent classes constructor to begin with 
        wx.Frame.__init__(self, parent, -1, title="Cloud Play Controls", pos=(3,3))
        panel = wx.Panel(self, -1)
        
        # the event caller
        self.eventCaller = eventCaller

        # cloud parameter details window
        cloudPresetFrame = cloudGui.CloudGUI(self, eventCaller)
        cloudPresetFrame.Show()
        self.eventCaller.guiCloudPreset = cloudPresetFrame
        
        # preset names
        self.currentCloudPreset = 'presets/template.clo'
        self.currentCloudSequence = 'presets/template.csq'
        self.loadWithListEnable = False
        
        # menu bar
        menuBar = wx.MenuBar()
        fileMenu = wx.Menu()
        fileMenu.Append(ID_FileMenuOpenCloud, "Open single cloud preset \tCtrl+O")
        fileMenu.Append(ID_FileMenuSaveCloud, "Save single cloud preset \tCtrl+S")
        fileMenu.Append(ID_FileMenuSaveCloudAs, "Save single cloud preset as .. \tCtrl+A")
        fileMenu.Append(ID_FileMenuOpenSeq, "Open sequence \tCtrl+Shift+O")
        fileMenu.Append(ID_FileMenuSaveSeq, "Save sequence \tCtrl+Shift+S")
        fileMenu.Append(ID_FileMenuSaveSeqAs, "Save sequence as ... \tCtrl+Shift+A")
        fileMenu.Append(ID_FileMenuExit, "Exit")
        menuBar.Append(fileMenu, "File")
        self.SetMenuBar(menuBar)
        # Menu events
        self.Bind(wx.EVT_MENU, self.MenuControls, id=ID_FileMenuOpenCloud)
        self.Bind(wx.EVT_MENU, self.MenuControls, id=ID_FileMenuSaveCloud)
        self.Bind(wx.EVT_MENU, self.MenuControls, id=ID_FileMenuSaveCloudAs)
        self.Bind(wx.EVT_MENU, self.MenuControls, id=ID_FileMenuOpenSeq)
        self.Bind(wx.EVT_MENU, self.MenuControls, id=ID_FileMenuSaveSeq)
        self.Bind(wx.EVT_MENU, self.MenuControls, id=ID_FileMenuSaveSeqAs)
        self.Bind(wx.EVT_MENU, self.MenuControls, id=ID_FileMenuExit)
    
        # define some GUI widgets, to be added to a Sizer later

        # Text Ctrls
        self.masterAmpCtrl = wx.TextCtrl(panel, ID_masterAmp, '1.0', (0,0), (34,20))
        wx.EVT_TEXT(self, ID_masterAmp, self.onTextCtrl)
        self.durationCtrl = wx.TextCtrl(panel, ID_Duration, "3.0", (0, 0), (34, 20))
        wx.EVT_TEXT(self, ID_Duration, self.onDuration)
        self.transpScaleCtrl = wx.TextCtrl(panel, ID_transpScale, "1.0", (0, 0), (34, 20))
        wx.EVT_TEXT(self, ID_transpScale, self.onTextCtrl)
        self.rateScaleCtrl = wx.TextCtrl(panel, ID_rateScale, "1.0", (0, 0), (34, 20))
        wx.EVT_TEXT(self, ID_rateScale, self.onTextCtrl)
        self.turbulenceCtrl = wx.TextCtrl(panel, ID_Turbulence, "0.0", (0, 0), (34, 20))
        wx.EVT_TEXT(self, ID_Turbulence, self.onTextCtrl)
        self.transparencyCtrl = wx.TextCtrl(panel, ID_Transparency, "0.0", (0, 0), (34, 20))
        wx.EVT_TEXT(self, ID_Transparency, self.onTextCtrl)
        self.reverbCtrl = wx.TextCtrl(panel, ID_cloudReverb, "0.0", (0, 0), (34,20))
        wx.EVT_TEXT(self, ID_cloudReverb, self.onTextCtrl)
        self.delayCtrl = wx.TextCtrl(panel, ID_cloudDelay, "0.0", (0, 0), (34,20))
        wx.EVT_TEXT(self, ID_cloudDelay, self.onTextCtrl)
        self.widthCtrl = wx.TextCtrl(panel, ID_cloudWidth, "0.0", (0, 0), (34,20))
        wx.EVT_TEXT(self, ID_cloudWidth, self.onTextCtrl)

        self.masterAmpLabel = wx.StaticText(panel, -1, "Amp")
        self.durationLabel = wx.StaticText(panel, -1, "Duration (Cloud vSize)", (0, 0))
        self.transpScaleLabel = wx.StaticText(panel, -1, "Transp Scale", (0, 0))
        self.rateScaleLabel = wx.StaticText(panel, -1, "Rate Scale", (0, 0))
        self.turbulenceLabel = wx.StaticText(panel, -1, "Turbulence", (0, 0))
        self.transparencyLabel = wx.StaticText(panel, -1, "Transparency", (0, 0))
        self.reverbLabel = wx.StaticText(panel, -1, "Reverb", (0, 0))
        self.delayLabel = wx.StaticText(panel, -1, "Delay", (0, 0))
        self.widthLabel = wx.StaticText(panel, -1, "St.width", (0, 0))
        self.waveformMenuLabel = wx.StaticText(panel, -1, "waveform menu 1-4", (0, 0))
        self.cloudPresetMenuLabel = wx.StaticText(panel, -1, "cloud preset", (0, 0))

        #check box
        self.loadWithListCtrl = wx.CheckBox(panel, ID_loadWithList, "listCtrl loads preset", (0,0))
        wx.EVT_CHECKBOX(self, ID_loadWithList, self.onLoadWithListToggle)

        # buttons
        self.playCtrl = wx.Button(panel, ID_Play, "Play single", (0, 0))
        wx.EVT_BUTTON(self, ID_Play, self.onPlay)       

        # a bitmap for "details" buttons
        self.moreIcon = wx.Bitmap('plus.bmp')
        self.showCloudPresetControls = wx.BitmapButton(panel, ID_ShowCloudPresetControls, self.moreIcon)
        wx.EVT_BUTTON(self, ID_ShowCloudPresetControls, self.onShowSubwindow)

        self.addToSeqCtrl = wx.Button(panel, ID_AddToSeq, "Add preset to Seq", (0, 0))
        wx.EVT_BUTTON(self, ID_AddToSeq, self.onAddToSeq)       
        
        self.cloudSequencePlayCtrl = wx.ToggleButton(panel, ID_cloudSequencePlay, "Play cloudSeq", (0, 0))
        wx.EVT_TOGGLEBUTTON(self, ID_cloudSequencePlay, self.onCloudSequencePlay)

        # list controls
        self.cloudWave1Ctrl = wx.ComboBox(panel, ID_cloudWave1, ' ', (0, 0), (100,20), choices=[])
        self.cloudWave2Ctrl = wx.ComboBox(panel, ID_cloudWave2, ' ', (0, 0), (100,20), choices=[])
        self.cloudWave3Ctrl = wx.ComboBox(panel, ID_cloudWave3, ' ', (0, 0), (100,20), choices=[])
        self.cloudWave4Ctrl = wx.ComboBox(panel, ID_cloudWave4, ' ', (0, 0), (100,20), choices=[])
        wx.EVT_COMBOBOX(self, ID_cloudWave1, self.onCloudWave)
        wx.EVT_COMBOBOX(self, ID_cloudWave2, self.onCloudWave)
        wx.EVT_COMBOBOX(self, ID_cloudWave3, self.onCloudWave)
        wx.EVT_COMBOBOX(self, ID_cloudWave4, self.onCloudWave)


        presetlist = ['no preset']
        self.cloudPresetMenuCtrl = wx.ComboBox(panel, ID_cloudPresetMenu, 'no preset', (0, 0), (100,20), choices=presetlist)
        wx.EVT_COMBOBOX(self, ID_cloudPresetMenu, self.onCloudPresetMenu)
       
        self.cloudSequenceCtrl = wx.gizmos.EditableListBox(panel, ID_cloudSequence, "cloud sequence", (0,0), (300, 300))

        # make a Sizer to hold widgets
        siz1vgap = 3
        siz1hgap = 8
        sizer1 = wx.GridBagSizer(siz1vgap, siz1hgap)
        sizer1.SetEmptyCellSize((5,5))
        
        # add the widgets to the Sizer

        sizer1.Add(self.playCtrl,           (1,1)) 
        sizer1.Add(self.showCloudPresetControls, (1,2)) 
        sizer1.Add(self.masterAmpLabel,     (2,1))
        sizer1.Add(self.masterAmpCtrl,      (2,2))
        sizer1.Add(self.durationLabel,      (3,1))
        sizer1.Add(self.durationCtrl,       (3,2)) 
        sizer1.Add(self.transpScaleLabel,   (4,1))
        sizer1.Add(self.transpScaleCtrl,    (4,2)) 
        sizer1.Add(self.rateScaleLabel,     (5,1))
        sizer1.Add(self.rateScaleCtrl,      (5,2)) 
        sizer1.Add(self.turbulenceLabel,    (6,1))
        sizer1.Add(self.turbulenceCtrl,     (6,2)) 
        sizer1.Add(self.transparencyLabel,  (7,1))
        sizer1.Add(self.transparencyCtrl,   (7,2)) 
        sizer1.Add(self.reverbLabel,        (8,1))
        sizer1.Add(self.reverbCtrl,         (8,2)) 
        sizer1.Add(self.delayLabel,         (9,1))
        sizer1.Add(self.delayCtrl,          (9,2)) 
        sizer1.Add(self.widthLabel,         (10,1))
        sizer1.Add(self.widthCtrl,          (10,2))
        
        sizer1.Add(self.waveformMenuLabel,  (11,1), (1,2), wx.ALIGN_CENTER)
        sizer1.Add(self.cloudWave1Ctrl,     (12,1))
        sizer1.Add(self.cloudWave2Ctrl,     (12,2))
        sizer1.Add(self.cloudWave3Ctrl,     (13,1))
        sizer1.Add(self.cloudWave4Ctrl,     (13,2))
        
        sizer1.Add((20,20),                 (14,1))
        
        sizer1.Add(self.cloudPresetMenuLabel, (15,2))
        sizer1.Add(self.addToSeqCtrl,         (16,1))
        sizer1.Add(self.cloudPresetMenuCtrl,  (16,2))
        sizer1.Add(self.cloudSequencePlayCtrl,(17,1))
        sizer1.Add(self.loadWithListCtrl,     (17,2))
        sizer1.Add(self.cloudSequenceCtrl,    (18,1), (2,2))

        # bottom spacer
        sizer1.Add((0, 20), (19,3))
        
        # set the size of the parent window1
        # to fit the space needed for the sizer
        sizer1.SetSizeHints(self)
        self.SetSizer(sizer1)
        #self.SetAutoLayout(1)
        #self.Layout()

    def onShowSubwindow(self, event):
        """
        Control show/hide of subwindows of the main GUI panel.
        
        @param self: The object pointer.
        @param event: The wx event.        
        """
        window = self.eventCaller.guiCloudPreset
        if window.IsShown():
            window.Raise()
            print 'window raised'
        else:
            window.Show(True)
            print 'window shown'

    def MenuControls(self, event):
        """
        Event processing from menubar gui controls.
        
        @param self: The object pointer.
        @param event: The wx event.        
        """
        eventID = event.GetId()
        if eventID == ID_FileMenuOpenCloud:
            self.cloudPresetOpen()
        elif eventID == ID_FileMenuSaveCloud:
            self.cloudPresetSave()
        elif eventID == ID_FileMenuSaveCloudAs:
            self.cloudPresetSaveAs()
        elif eventID == ID_FileMenuOpenSeq:
            self.cloudSequenceOpen()
        elif eventID == ID_FileMenuSaveSeq:
            self.cloudSequenceSave()
        elif eventID == ID_FileMenuSaveSeqAs:
            self.cloudSequenceSaveAs()
        elif eventID == ID_FileMenuExit:
            print "exit app"
            self.Close()
        else:
            print 'menubar: unknown item', eventID, event

    def onPlay(self, event):
        """
        Called when the "Play single"button is used.
        
        @param self: The object pointer.
        @param event: The wx event.        
        """

        self.eventCaller.playCloud()

    def onCloudSequencePlay(self, event):
        """
        Called when the "Play cloudSeq" button is used.
        
        @param self: The object pointer.
        @param event: The wx event.        
        """
        if event.IsChecked() == 1:
            self.eventCaller.setCloudSequenceContinueFlag(1)
            self.eventCaller.cloudSequencePlay()
            #self.eventCaller.cloudVolume('on')
        else:
            self.eventCaller.setCloudSequenceContinueFlag(0)
            #self.eventCaller.cloudVolume('off')
        
    def onCloudPresetMenu(self, event):
        """
        Called when the Cloud preset menu control is used.
        
        @param self: The object pointer.
        @param event: The wx event.        
        """
        if self.loadWithListEnable:
            if (event.GetString() == 'static') or (event.GetString() == 'interpolate'):
                print 'note: static and interpolate presets can only be used in cloud sequences'
                return
            # check if the current preset is edited before opening file
            if self.eventCaller.guiCloudPreset.isEdited:
                dlg = wx.MessageDialog(self, 'Unsaved changes to current preset, discard changes ?', 'Warning',
                                       wx.YES_NO | wx.NO_DEFAULT | wx.ICON_EXCLAMATION)
                choose = dlg.ShowModal()
                dlg.Destroy()
                if choose == wx.ID_NO:
                    print 'file open cancelled, save your changes and try again'
                    return
            self.eventCaller.cloudPlayer.makePresetCurrent(event.GetString())
            self.currentCloudPreset = 'presets/' + event.GetString() + '.clo'
            self.eventCaller.guiCloudPreset.setWindowTitle('Cloud Preset Controls          loaded preset  :  ' + event.GetString())
            self.eventCaller.guiCloudPreset.isEdited = False
            # update GUI
            self.eventCaller.updateCloudPresetGui()

    def onAddToSeq(self, event):
        """
        Called when the AddToSeq button is used.
        
        @param self: The object pointer.
        @param event: The wx event.        
        """
        alist = self.cloudSequenceCtrl.GetStrings()
        alist.append("%s, %s" %(self.cloudPresetMenuCtrl.GetValue(), self.durationCtrl.GetValue()))
        self.eventCaller.cloudSequenceList = alist
        self.cloudSequenceCtrl.SetStrings(alist)
        
    def cloudPresetOpen(self):
        """
        Recall parameter preset from file.
        
        File name selected in user dialog.
        
        @param self: The object pointer.
        """        
        # check if the current preset is edited before opening file
        if self.eventCaller.guiCloudPreset.isEdited:
            dlg = wx.MessageDialog(self, 'Unsaved changes to current preset, discard changes ?', 'Warning',
                                   wx.YES_NO | wx.NO_DEFAULT | wx.ICON_EXCLAMATION)
            choose = dlg.ShowModal()
            dlg.Destroy()
            if choose == wx.ID_NO:
                print 'file open cancelled, save your changes and try again'
                return
        
        dlg = wx.FileDialog(
            self, message="Open single cloud preset", defaultDir='presets/',  
            defaultFile="", wildcard="Single Cloud Preset (*.clo)|*.clo", style=wx.OPEN
            )

        # Show the dialog and retrieve the user response. If it is the OK response, 
        # process the data.
        if dlg.ShowModal() == wx.ID_OK:
            filename = dlg.GetPath()

            try:
                n = filename.find('presets')
                filename = filename[n+8:]
            except:
                print 'do keep all presets in the presets directory, file operation terminated without success'
                return
                        
            presetName = self.eventCaller.cloudPlayer.loadSingleCloudPresetFromFile(filename)
            self.eventCaller.cloudPlayer.makePresetCurrent(presetName)
            self.currentCloudPreset = 'presets/' + filename
            self.eventCaller.guiCloudPreset.setWindowTitle('Cloud Preset Controls          loaded preset  :  ' + presetName)
            self.eventCaller.guiCloudPreset.isEdited = False
            # update GUI
            self.eventCaller.updateCloudPresetGui()

        # Destroy the dialog. Don't do this until you are done with it!
        # BAD things can happen otherwise!
        dlg.Destroy()

    def cloudPresetSaveAs(self):
        """
        Save parameter preset to file (save as...).
        
        @param self: The object pointer.
        """        
        dlg = wx.FileDialog(
            self, message="Save single cloud preset as ...", defaultDir='presets/', 
            defaultFile="", wildcard="Single Cloud Preset (*.clo)|*.clo", style=wx.SAVE
            )
        # Show the dialog and retrieve the user response. If it is the OK response, 
        # process the data.
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            self.cloudPresetSave(path)
                
        # Destroy the dialog. Don't do this until you are done with it!
        # BAD things can happen otherwise!
        dlg.Destroy()

    def cloudPresetSave(self, filename=None):
        """
        Save parameter preset to file.
        
        @param self: The object pointer.
        @parameter filename: The file name to save to.
        """        
        if not filename:
            filename = self.currentCloudPreset
            filename = os.getcwd() + '/' + filename
            print 'test', filename
        if os.path.exists(filename):
            dlg = wx.MessageDialog(self, 'File exist, overwrite ?', 'Warning',
                                   wx.OK | wx.CANCEL | wx.ICON_EXCLAMATION)
            choose = dlg.ShowModal()
            dlg.Destroy()
            if choose == wx.ID_CANCEL:
                print 'preset save cancelled'
                return
        f = file(filename, 'w')
        for key,value in self.eventCaller.cloudPlayer.parameters.iteritems():
            linestring = key + ', ' + str(value) + '\n'
            f.write(linestring)
        f.close
        presetName = filename[filename.find('presets')+8:].rstrip('.clo')
        self.currentCloudPreset = filename[filename.find('presets'):]
        print 'self.currentCloudPreset', self.currentCloudPreset
        self.eventCaller.guiCloudPreset.setWindowTitle('Cloud Preset Controls          loaded preset  :  ' + presetName)
        self.eventCaller.guiCloudPreset.isEdited = False
        # update the data in cloudPlayer's internal data memory (dictionary)
        isNewPreset = self.eventCaller.cloudPlayer.updateSingleCloudPreset(presetName)
        if isNewPreset:
            self.cloudPresetMenuCtrl.Append(presetName)
        print filename, 'saved'
        
    def cloudSequenceOpen(self):
        """
        Recall a sequence of several parameter presets from file.
        
        Sequence file name selected in user dialog.
        
        @param self: The object pointer.
        """        
        dlg = wx.FileDialog(
            self, message="Open cloud sequence", defaultDir='presets/',  
            defaultFile="", wildcard="Cloud Sequence Preset (*.csq)|*.csq", style=wx.OPEN
            )

        # Show the dialog and retrieve the user response. If it is the OK response, 
        # process the data.
        if dlg.ShowModal() == wx.ID_OK:
            filename = dlg.GetPath()
            '''
            try:
                n = filename.find('presets')
                filename = filename[n+8:].rstrip('.csq')
            except:
                print 'do keep all presets in the presets directory, file operation terminated without success'
                return
            '''            
            self.eventCaller.cloudSequenceRecall(filename)
            self.currentCloudSequence = filename
            print 'opened', filename
        # Destroy the dialog. Don't do this until you are done with it!
        # BAD things can happen otherwise!
        dlg.Destroy()

    def cloudSequenceSaveAs(self):
        """
        Save a sequence of several parameter presets to file (save as...).
                
        @param self: The object pointer.
        """        
        dlg = wx.FileDialog(
            self, message="Save cloud sequence as ...", defaultDir='presets/', 
            defaultFile="", wildcard="Cloud Sequence Preset (*.csq)|*.csq", style=wx.SAVE
            )
        # Show the dialog and retrieve the user response. If it is the OK response, 
        # process the data.
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            
            if os.path.exists(path):
                dlg = wx.MessageDialog(self, 'File exist, overwrite ?', 'Warning',
                                       wx.OK | wx.CANCEL | wx.ICON_EXCLAMATION)
                choose = dlg.ShowModal()
                dlg.Destroy()
                if choose == wx.ID_OK:
                    self.cloudSequenceSave(path)
            else:
                self.cloudSequenceSave(path)
                
        # Destroy the dialog. Don't do this until you are done with it!
        # BAD things can happen otherwise!
        dlg.Destroy()

    def cloudSequenceSave(self, filename=None):
        """
        Save a sequence of several parameter presets to file.
                
        @param self: The object pointer.
        @param filename: The file name to save to.
        """        
        if not filename:
            filename = self.currentCloudSequence
        self.eventCaller.cloudSequenceSave(self.cloudSequenceCtrl.GetStrings(), filename)
        print filename, 'saved'


    def onTextCtrl(self, event):
        """
        Called when any of the gui text controls are used.
        
        @param self: The object pointer.
        @param event: The wx event.        
        """
        eventID = event.GetEventObject().GetId()
        if eventID == ID_masterAmp: name = 'MasterLevel'
        if eventID == ID_transpScale: name = 'TranspScale'
        if eventID == ID_rateScale: name = 'RateScale'
        if eventID == ID_Turbulence: name = 'Turbulence'
        if eventID == ID_Transparency: name = 'Transparency'
        if eventID == ID_cloudReverb: name = 'CloudReverbAmt'
        if eventID == ID_cloudDelay: name = 'CloudDelayAmt'
        if eventID == ID_cloudWidth: name = 'CloudWidth'
        self.eventCaller.sendControlToCsound(name, float(event.GetString()))

    def onDuration(self, event):
        """
        Called when the duration gui control is used.
        
        @param self: The object pointer.
        @param event: The wx event.        
        """
        self.eventCaller.setCloudDuration(float(event.GetString()))

    def onLoadWithListToggle(self, event):
        """
        Called when the LoadWithListToggle widget is used.
        
        @param self: The object pointer.
        @param event: The wx event.        
        """
        self.loadWithListEnable = event.IsChecked()

    def onCloudWave(self, event):
        """
        Called when an item of the cloudWave comboBox is selected
        it selects the waveform (wave file) used in particle clouds
        
        @param self: The object pointer.
        @param event: The name of the wave file.
        """
        # Retrieves the ID of the comboBox that was used
        eventID = event.GetEventObject().GetId()
        print 'onCloudWave called', eventID, event.GetString()
        waveNum = eventID - ID_cloudWave1 + 1
        self.eventCaller.cloudWaveSelect(waveNum, str(event.GetString()))

    def updateCloudSequenceList(self, cloudData):
        """
        Updates the contents of the list box with the list of items in the argument.
        
        @param self: The object pointer.
        @param cloudData: The list of items to write to the GUI control.
        """
        #print 'gui.cloudSequenceCtrl updated', cloudData
        self.cloudSequenceCtrl.SetStrings(cloudData)

    def updateCloudPresetComboBox(self, presetList):
        """
        Update the menu object in the Gui to reflect the presets in memory.
        
        @param self: The object pointer.
        @parame presetList: The list of preset to be put in the menu (aka choice list).
        """
        # clear the combo box
        self.cloudPresetMenuCtrl.Clear()
        self.cloudPresetMenuCtrl.Append('static')
        self.cloudPresetMenuCtrl.Append('interpolate')
        for item in presetList:
            # append the item to the combo box
            self.cloudPresetMenuCtrl.Append(item)
                                
    def updateCloudWaveComboBox(self, files):
        """
        Update the menu object in the Gui to reflect the particle wave files in memory.
        
        @param self: The object pointer.
        @param files: The list of files to be put in the menu (aka choice list).
        """
        for f in files:
            # append the item to the combo box
            self.cloudWave1Ctrl.Append(f)
            self.cloudWave2Ctrl.Append(f)
            self.cloudWave3Ctrl.Append(f)
            self.cloudWave4Ctrl.Append(f)

    def updateCloudWave(self, waveNum, waveName):
        """
        Update the displayed wave in the cloud wave combo box.
        
        @param self: The object pointer.
        @param waveNum: The combo box to be updated, corresponds to particle wave channel.
        @param waveName: The name of the wave to select in the combo box.
        """
        # first format a string corresponding to the gui control method
        ctrl = 'self.cloudWave%iCtrl' %waveNum
        #print '**** *** gui.updateCloudWave: ctrl = %s *** *** ***' %ctrl
        # eval the string to point to the actual gui control
        guiControl = eval(ctrl)
        # execute call to update gui control
        guiControl.SetValue(waveName)
                                    
def test():
    """
    self test
    """
    ctrl_app = wx.PySimpleApp()
    eventCaller = ''
    frame = PlayControls(None, eventCaller)
    frame.Show(True)
    ctrl_app.MainLoop()

if __name__ == '__main__':
    test()
